*! version 1.0.4  26nov2006  by Marc Muendler muendler@ucsd.edu

program define est2extlbl
  version 7
  syntax [, ADDto(string) REPLACE Saving DROPALL Path(string) INtex(string) RETain]

  if "$S_OS" == "MacOS" {
    local dirsep = ":"
    }
  else {
    local dirsep = "/"
    }  
  if (substr("`path'",-1,1)~="\" & substr("`path'",-1,1)~="/" & substr("`path'",-1,1)~=":") & "`path'"~="" {
    local path "`path'`dirsep'"
    }
  local outfile "`path'`addto'_rowlbl.dta"
  if "`replace'"~="replace" {
    confirm new file "`outfile'"
    }
  if "`addto'"=="" {
    disp as err "File addto expected"
    error 7
    }
  if "`intex'"=="" & "`retain'"=="" {
    disp as err "Either intex() file or retain option must be specified"
    error
    }
  if substr("`intex'",-4,4)~=".tex" {
    local intex "`intex'.tex"
    }
  local infile "`intex'"
  if substr("`outdta'",-7,7)~="_rowlbl" & substr("`outdta'",-11,11)~="_rowlbl.dta" {
    local outdta "`outdta'_rowlbl.dta"
    }
  if "`retain'"~="" {
    confirm string variable _rowname _rowlbl
    }

  local saved 0
  if "`retain'"=="" {
    local currN = _N
    if `currN'>0 & "`saving'"~="saving" & "`dropall'"~="dropall" {
      error 4
      exit
      }
    if "`saving'"=="saving" & `currN'>0 {
      local dropall = ""
      tempfile tmp
      quietly save `tmp'
      local saved 1
      disp in green "data in memory will be removed temporarily, and restored later"
      disp in green "Press any key to continue, or Break to abort"
      more
      drop _all
      }
    confirm file "`infile'"
    if "`dropall'"=="dropall" & `currN'>0 {
      disp in green "data in memory will be dropped"
      disp in green "Press any key to continue, or Break to abort"
      more
      drop _all
      }
    capture {
      insheet using "`infile'", delimiter("&")
      capture drop if substr(v1,1,1)=="%"
      rename v1 _rowname
      rename v2 _rowlbl
      capture confirm new variable v3
      if _rc~=0 {
        disp as err "additional column found in TeX table where two columns expected"
        list if v3~=""
        disp as err "(_rowlbl likely includes ampersand &)"
        error 7
        }
      capture replace _rowname = trim(_rowname)
      replace _rowlbl = substr(_rowlbl,1,length(_rowlbl)-1) if substr(_rowlbl,-1,1)=="\" /* remove single "\" from end of string */
      replace _rowlbl = substr(_rowlbl,1,length(_rowlbl)-1) if substr(_rowlbl,-1,1)=="\" /* remove line break "\\" from string */
      replace _rowlbl = trim(_rowlbl)
      drop if _rowname=="" & _rowlbl==""
      qui count if index(_rowlbl,"$")>0 & lower(substr(_rowlbl,index(_rowlbl,"$")+1,1))>="a" & lower(substr(_rowlbl,index(_rowlbl,"$")+1,1))<="z"
      sort _rowname
      if `r(N)' > 0 {
        noi disp in blue "Warning. There are TeX math mode $ signs followed by letters."
        noi list if index(_rowlbl,"$")>0 & lower(substr(_rowlbl,index(_rowlbl,"$")+1,1))>="a" & lower(substr(_rowlbl,index(_rowlbl,"$")+1,1))<="z"
        noi disp in blue "Stata will consider these math mode $ signs as global macro calls."
        noi disp in blue "Add blanks after the math mode $ signs if needed."
        noi disp
        }
      qui compress
      }
    }
  
  qui save "`outfile'", replace
  if _rc==0 {
    disp in green "list of row labels saved to " in yellow "`outfile'"
    }
  if `saved' {
    use `tmp', clear
    disp in green "data in memory restored"
    }

end

program define est2one
    version 8
    args vecname
    local vecname= subinstr("`vecname'",",","",1)
    capture syntax newvarname [, MULTSE(string) HORIzontal CUT ]
    if _rc==100 {
       disp as err "matrix initial required" 
       exit _rc
       }
    if _rc~=0 error _rc

    tempname chk
    capture local `chk' = colsof(`vecname'_tbl)
    if _rc==0 {
      disp as err "no; matrix " in yellow "`vecname'_tbl" in red " would be lost"
      exit 4 
      }
    if "`multse'" ~= "" {
        tokenize "`multse'"
        local msenum : word count `multse'
        forvalues i = 1/`msenum' {
            capture confirm integer number ``i''
            if _rc~=0 {
              disp as err "``i'' found where integer exected in multse()"
              exit _rc
              }
            }
        }
    if "`horizontal'"=="horizontal"  local cut = "cut"

    /* Check existence and conformability */
    local colmax = colsof(`vecname'_b)
    if "`multse'"==""  local try = colsof(`vecname'_se)
    else {
      foreach mse in `multse' {
         local try    = colsof(`vecname'_se`mse')
      }
    }
    local e_exist 0
    local r_exist 0
    capture local try = colsof(`vecname'_e)
    if _rc==0  local e_exist 1
    capture local try = colsof(`vecname'_r)
    if _rc==0  local r_exist 1
    macro drop try
    if "`multse'"==""  {
        capture matrix _tmp= `vecname'_b , `vecname'_se
        if _rc==503 {
                    disp as err "conformability error in rows of `vecname'_b and `vecname'_se"
                    exit _rc
                    }
        if _rc~=0  error _rc
        capture matrix _tmp= `vecname'_b \ `vecname'_se
        if _rc==503 {
                    disp as err "conformability error in columns of `vecname'_b and `vecname'_se"
                    exit _rc
                    }
        if _rc~=0  error _rc
    }
    else {
      foreach mse in `multse' {
        capture matrix _tmp= `vecname'_b , `vecname'_se`mse'
        if _rc==503 {
                    disp as err "conformability error in rows of `vecname'_b and `vecname'_se`mse'"
                    exit _rc
                    }
        if _rc~=0  error _rc
        capture matrix _tmp= `vecname'_b \ `vecname'_se`mse'
        if _rc==503 {
                    disp as err "conformability error in columns of `vecname'_b and `vecname'_se`mse'"
                    exit _rc
                    }
        if _rc~=0  error _rc
      }
    }
    matrix drop _tmp

    /* Transpose if option horizontal is chosen */
    if "`horizontal'" == "horizontal" {
        matrix `vecname'_b = `vecname'_b '
        if "`multse'"==""  matrix `vecname'_se = `vecname'_se '
        else {
            foreach mse in `multse' {
                matrix `vecname'_se`mse' = `vecname'_se`mse' '
            }       
        }
    }
    
    /* Create vertical master table */
    local msenum : word count `multse'
    local msfact = `msenum'+1
    if "`multse'"==""  matrix `vecname'_tbl = J(rowsof(`vecname'_b)*2,colsof(`vecname'_b),.)
    else  matrix `vecname'_tbl = J(rowsof(`vecname'_b)*`msfact',colsof(`vecname'_b),.)
    local colname : colfullnames `vecname'_b
    matrix colnames `vecname'_tbl = `colname'
    local coleq : coleq `vecname'_b
    matrix coleq `vecname'_tbl = `coleq'
    
    local rowname : rowfullnames `vecname'_b
    tokenize "`rowname'"
    local rowmax = rowsof(`vecname'_b)
    local colmax = colsof(`vecname'_b)
    if "`multse'"==""  {
      forvalues colj = 1/`colmax' {
        forvalues rowi = 1/`rowmax' {
            matrix `vecname'_tbl[`rowi'*2-1,`colj'] =  `vecname'_b[`rowi',`colj']
            local rowrepl = `rowi'*2-1
            matname `vecname'_tbl ``rowi'' , rows(`rowrepl') explicit
            matrix `vecname'_tbl[`rowi'*2  ,`colj'] = `vecname'_se[`rowi',`colj']
            local rowrepl = `rowi'*2
            matname `vecname'_tbl _se , rows(`rowrepl') explicit
            if `vecname'_b[`rowi',`colj']==0 & `vecname'_se[`rowi',`colj']==0  matrix `vecname'_tbl[`rowi'*2-1,`colj'] = .
            if `vecname'_se[`rowi',`colj']==0  matrix `vecname'_tbl[`rowi'*2,`colj'] = .
        }
      }
    }
    else {
      forvalues colj = 1/`colmax' {
        forvalues rowi = 1/`rowmax' {
            matrix `vecname'_tbl[`rowi'*`msfact'-`msenum',`colj'] =  `vecname'_b[`rowi',`colj']
            local rowrepl = `rowi'*`msfact'-`msenum'
            matname `vecname'_tbl ``rowi'' , rows(`rowrepl') explicit
            local msecnt = 0
            foreach mse in `multse' {
                local msecnt = `msecnt'+1
                matrix `vecname'_tbl[`rowi'*`msfact'-(`msenum'-`msecnt'),`colj'] = `vecname'_se`mse'[`rowi',`colj']
                local rowrepl = `rowi'*`msfact'-(`msenum'-`msecnt')
                matname `vecname'_tbl _se`mse' , rows(`rowrepl') explicit
                if `vecname'_b[`rowi',`colj']==0 & `vecname'_se`mse'[`rowi',`colj']==0 {
                    matrix `vecname'_tbl[`rowi'*`msfact'-`msenum',`colj'] = .
                    local msecnt = 0
                    foreach mse in `multse' {
                        local msecnt = `msecnt'+1                
                        matrix `vecname'_tbl[`rowi'*`msfact'-(`msenum'-`msecnt'),`colj'] = .
                        }
                }
            }
        }
      }
    }
    
    if "`cut'" ~= "cut" {
        /* Add _e and _r */
        if `e_exist'  capture matrix `vecname'_tbl = `vecname'_tbl \ `vecname'_e
        if _rc==503 {
          disp as err "conformability error for `vecname'_e"
          exit _rc
          }
        else  error _rc
        if `r_exist'  capture matrix `vecname'_tbl = `vecname'_tbl \ `vecname'_r
        if _rc==503 {
          disp as err "conformability error for `vecname'_r"
          exit _rc
          }
        else  error _rc
        }
    
    /* Transpose back if necessary */
    if "`horizontal'" == "horizontal" {
        matrix `vecname'_b = `vecname'_b '
        if "`multse'"==""  matrix `vecname'_se = `vecname'_se '
        else {
            foreach mse in `multse' {
                matrix `vecname'_se`mse' = `vecname'_se`mse' '
            }       
        }
    }
    
end

program define est2tex
    version 8
    args vecname

    capture syntax newvarname [, REPLACE DROPALL PREServe Path(string) MULTSE(string) Mark(string) LEVels(string) MARKTBL PREcision(real 0) DIGits(integer 3) FLExible(integer 2) FANcy HORIzontal CUT LABel COLLABels(string) VALLABels(string) EXtracols(string) DOTs SUPPRESS PLAIN(string) READY LEADZERO THOUSEP]
    if _rc==100 {
       disp as err "matrix name required" 
       exit _rc
       }
    if _rc~=0 {
      error _rc
      } 

    local vecname= subinstr("`vecname'",",","",1)
    if "$S_OS" == "MacOS" {
      local dirsep = ":"
      }
    else {
      local dirsep = "/"
      }
    if (substr("`path'",-1,1)~="\" & substr("`path'",-1,1)~="/" & substr("`path'",-1,1)~=":") & "`path'"~="" {
      local path "`path'`dirsep'"
      }
    local file1 "`path'`vecname'_tbl.dta"
    if "`plain'"==""    {
      local file2 "`path'`vecname'.tex"
      }
    if "`plain'"=="csv" {
      local file2 "`path'`vecname'.csv"
      }
    if "`plain'"=="tab" {
      local file2 "`path'`vecname'.txt"
      }


    /* Check for errors */

    if "`replace'"~="replace" {
      confirm new file "`file1'"
      confirm new file "`file2'"
      }
    if `digits'~=float(0) & `digits'~=float(3) & `precision'~=float(0) {
      disp as err "digits() and precision() invalid"
      exit 198
      }
    local positse = -3
    local linesup = 1
    if "`multse'"~="" & "`mark'"~="" {
        local positse = -4
        tokenize "`mark'"
        local markse = "`2'"
        tokenize "`multse'"
        local maxx = wordcount("`multse'")
        local linesup = 0
        forvalues x = 1/`maxx' {
          if ``x'' == `markse' {
            local linesup = `x'
            }
          }
        if real("`markse'") == . {
            disp as err "multse() and mark() without reference invalid"
            exit 198
            }
        }
    if "`ready'"=="ready" {
        local colchk : colfullnames `vecname'_tbl
        local cols = colsof(`vecname'_tbl)
        tokenize `colchk'
        forvalues i = 1/`cols' {
          if real(substr("``i''",1,1))~=. {
            disp as err "at least one column name starts with integer number"
            exit 198
            }
          }
        }
    if "`ready'"=="ready" & "`mark'"~="" {
        local colchk : colfullnames `vecname'_tbl
        local cols = colsof(`vecname'_tbl)
        tokenize `colchk'
        forvalues i = 1/`cols' {
          if "``i''"~=subinstr("``i''","(","",.) | "``i''"~=subinstr("``i''",")","",.) | "``i''"~=subinstr("``i''","[","",.) | "``i''"~=subinstr("``i''","]","",.) {
            disp as err "column names contain invalid symbols for use with ready and mark()"
            exit 198
            }
          }
        tokenize "`mark'"
        local markse = "`2'"
        if real("`markse'") ~= . {
            disp as err "ready and mark() with reference invalid"
            exit 198
            }
        disp in blu "mark() option will not work properly if `vecname'_tbl contains multiple se"
        more
        }
    if "`levels'"=="" {
        if $S_level>99.9                 local levels = "99 $S_level 99.99"
        if $S_level>99 & $S_level<=99.9  local levels = "99 $S_level 99.9"
        if $S_level>98 & $S_level<=99    local levels = "95 $S_level 99.5"
        if $S_level>95 & $S_level<=98    local levels = "95 $S_level 99"
        if $S_level>90 & $S_level<=95    local levels = "90 $S_level 99"
        if $S_level<=90                  local levels = "$S_level 90 95"
        }
    tokenize "`levels'" 
    forvalues i = 1/3 { 
        capture confirm number ``i''
        if _rc~=0 { 
          if "``i''"==""  local `i' = "no entry"
          disp as err "levels() invalid. ``i'' found where number expected" 
          exit _rc
          }
        if ``i''<=0 | ``i''>=100 {
          disp as err "levels() invalid. ``i'' found where number between 0 and 100 expected" 
          exit 7
          }
        local lvl`i' = (100-``i'')/200 
        }
    if "`ready'"=="ready" & "`multse'"~="" {
        disp as err "ready and multse() invalid"
        exit 198
        }
    if `precision'~=0 {
        local precision = `precision' - int(`precision')
        local digits = length(string(`precision'))-1
        }
    if `digits'~=0 {
      local precision = 10^(-`digits')
      }
    local addzeros = ""
    forvalues i = 1/`digits' {
      local addzeros = "`addzeros'" + "0"
      }
    if substr("`mark'",1,5)~="stars" & substr("`mark'",1,5)~="starb" & substr("`mark'",1,2)~="it"  local mark = ""
    local toknum 0
    if "`extracols'" ~= "" {
        tokenize "`extracols'"
        local toknum : word count `extracols'
        forvalues i = 1/`toknum' {
            capture confirm integer number ``i''
            if _rc~=0 | ``i''<=0 {
              disp as err "``i'' found where positive integer expected in extracols()"
              capture confirm integer number .5
              exit _rc
              }
        }
    }
    if "`multse'" ~= "" {
        tokenize "`multse'"
        local msenum : word count `multse'
        forvalues i = 1/`msenum' {
            if ``i'' > 9 | ``i'' < 1 {
              local `i' = ``i'' + .5
              }
            capture confirm integer number ``i''
            if _rc~=0 {
              local `i' = int(``i'')
              disp as err "``i'' found where single-digit integer expected in multse()"
              exit _rc
              }
        }
    }
    if "`plain'"~="" & "`plain'"~="tab" & "`plain'"~="csv" {
        disp as err "option plain allows tab or csv"
        exit 198
        }
    if "`leadzero'"~="" & "`fancy'"~="fancy" {
        disp as err "option leadzero not allowed without option fancy"
        exit 198
        }
    if "`thousep'"~="" & "`fancy'"~="fancy" {
        disp as err "option thousep not allowed without option fancy"
        exit 198
        }
    if "`replace'"=="replace" & "`ready'"~="ready" {
      capture matrix drop `vecname'_tbl
      }
    if "`horizontal'" == "horizontal" | "`ready'"=="ready" { 
      local label = "" 
      local collabels = ""
      } 
    if "`label'"=="label" {   /* MH 16jan2002, adapted: MAM 26jun2002 */
        local i = 1
        local vars : rownames `vecname'_b
        foreach var in `vars' {
            capture confirm variable `var'
            if _rc==0 { 
              local L`i' : variable label `var' 
              }
            if "`L`i''"=="" { 
              local L`i' = "`var'" 
              }
            local i = `i'+1
            }
        }
    if "`collabels'"~="" {
        tokenize `collabels'
        local colmax = colsof(`vecname'_b)
        forvalues i = 1/`colmax' {
            local labl`i' = ""
            local labl`i' = "``i''"
            if subinword("`collabels'","``i''","",.)~=subinword("`collabels'","``i''","",1) {
                disp as err "names in collabels() are not unique"
                error 110
                }
            }
        }
    if "`preserve'"=="preserve" {   /* MH 16jan2002 */
        local dropall "dropall"
        preserve
        }
    if "`dropall'"=="dropall" {
      drop _all
      } 
    if _N>0 & "`preserve'"~="preserve" {
      error 4
      }
    if _N>0 {
      qui drop _all
      }
    if "`horizontal'"=="horizontal" {
      local cut = "cut"
      }
    if "`vallabels'"~=""  qui do "`path'`vallabels'"


    /* Start working */

    local lblfile "`path'`vecname'_rowlbl.dta"
    capture use "`lblfile'"
    if _rc==0 {
      local label = "label"
      local lblnum = _N
      forvalues l = 1/`lblnum' {
        local V`l' = _rowname[`l']
        local L`l' = _rowlbl[`l']
        if "`L`l''"== "" { 
          local L`l' = "`V`l''" 
          }
        }
      qui drop _all
      }
    if "`ready'"~="ready" { 
      est2one `vecname', `cut' `horizontal' multse(`multse') 
      }
    local colmax = colsof(`vecname'_tbl)
    local colnm : colfullnames `vecname'_tbl
    local chkcols : list uniq colnm
    local chkle : word count `chkcols'
    if `chkle' < `colmax' {
      disp as err "columns of " in yellow "`vecname'" in red " are not uniquely named"
      error 110
      }
    tokenize `colnm'
    forvalues i = 1/`colmax' {
      if "``i''"=="_all" | "``i''"=="all" {
        disp as err "column name " in yellow "``i''" in red " not permitted (system name conflict)"
        exit 198
        }
      }
  
    /* Create table of significance levels */

    if substr("`mark'",1,2)=="it" | substr("`mark'",1,4)=="star" { 
        local msenum : word count `multse'
        local msfact = `msenum'+1
        if "`ready'"~="ready" {
            if "`horizontal'"~="horizontal" {
                local rowsz = rowsof(`vecname'_b)
                local colsz = colsof(`vecname'_b)
            }
            else {
                local rowsz = colsof(`vecname'_b)
                local colsz = rowsof(`vecname'_b)          
            }
            local rowmax = `rowsz'
            local colmax = `colsz'
            }
        else {
            local rowsz = rowsof(`vecname'_tbl)
            local colsz = colsof(`vecname'_tbl)
            local colmax = `colsz'
            local rowmax = 0
            local rowload : rownames `vecname'_tbl
            tokenize "`rowload'"
            forvalues i = 1/`rowsz' {
                if substr("``i''",-3,3)=="_se" | substr("``i''",-4,3)=="_se"  local rowmax = `rowmax' + 1
                }
            }
        if "`marktbl'" == ""  tempname t_stars
        else  local t_stars = "t_stars"
        if "`multse'"==""   matrix `t_stars' = J(`rowsz'*2,`colsz',0)
        else  matrix `t_stars' = J(`rowsz'*`msfact',`colsz',0)
        local colnm : colfullnames `vecname'_tbl
        matrix colnames `t_stars' = `colnm'
        local rowsstar = min(rowsof(`t_stars'),rowsof(`vecname'_tbl))
        tempname justrows
        matrix `justrows' = `vecname'_tbl[1..`rowsstar',1]
        local rownm : rowfullnames `justrows'
        matrix rownames `t_stars' = `rownm'
        if "`ready'"~="ready" {
            if "`multse'"==""  {
                if "`horizontal'" == "horizontal" {
                  matrix `vecname'_b  = `vecname'_b '
                  matrix `vecname'_se = `vecname'_se'
                  }
                forvalues colj = 1/`colmax' {
                  forvalues rowi = 1/`rowmax' {
                    if abs(`vecname'_b[`rowi',`colj']/`vecname'_se[`rowi',`colj']) > abs(invnorm(`lvl1')) & abs(`vecname'_b[`rowi',`colj']/`vecname'_se[`rowi',`colj'])~=.  matrix `t_stars'[`rowi'*2-1,`colj']=1
                    if abs(`vecname'_b[`rowi',`colj']/`vecname'_se[`rowi',`colj']) > abs(invnorm(`lvl2')) & abs(`vecname'_b[`rowi',`colj']/`vecname'_se[`rowi',`colj'])~=.  matrix `t_stars'[`rowi'*2-1,`colj']=2
                    if abs(`vecname'_b[`rowi',`colj']/`vecname'_se[`rowi',`colj']) > abs(invnorm(`lvl3')) & abs(`vecname'_b[`rowi',`colj']/`vecname'_se[`rowi',`colj'])~=.  matrix `t_stars'[`rowi'*2-1,`colj']=3
                    if `vecname'_se[`rowi',`colj']==. & substr("`mark'",1,2)=="it"  matrix `t_stars'[`rowi'*2-1,`colj']=9
                  }
                }
                if "`horizontal'" == "horizontal" {
                  matrix `vecname'_b  = `vecname'_b '
                  matrix `vecname'_se = `vecname'_se'
                  }
            }
            else {
                if "`horizontal'" == "horizontal" {
                  matrix `vecname'_b          = `vecname'_b '
                  matrix `vecname'_se`markse' = `vecname'_se'`markse'
                  }
                forvalues colj = 1/`colmax' {
                  forvalues rowi = 1/`rowmax' {
                    if abs(`vecname'_b[`rowi',`colj']/`vecname'_se`markse'[`rowi',`colj']) > abs(invnorm(`lvl1')) & abs(`vecname'_b[`rowi',`colj']/`vecname'_se`markse'[`rowi',`colj'])~=.  matrix `t_stars'[`rowi'*`msfact'-`msenum',`colj']=1
                    if abs(`vecname'_b[`rowi',`colj']/`vecname'_se`markse'[`rowi',`colj']) > abs(invnorm(`lvl2')) & abs(`vecname'_b[`rowi',`colj']/`vecname'_se`markse'[`rowi',`colj'])~=.  matrix `t_stars'[`rowi'*`msfact'-`msenum',`colj']=2
                    if abs(`vecname'_b[`rowi',`colj']/`vecname'_se`markse'[`rowi',`colj']) > abs(invnorm(`lvl3')) & abs(`vecname'_b[`rowi',`colj']/`vecname'_se`markse'[`rowi',`colj'])~=.  matrix `t_stars'[`rowi'*`msfact'-`msenum',`colj']=3
                    if `vecname'_se`markse'[`rowi',`colj']==. & substr("`mark'",1,2)=="it"  matrix `t_stars'[`rowi'*`msfact'-`msenum',`colj']=9
                  }
                }           
                if "`horizontal'" == "horizontal" {
                  matrix `vecname'_b          = `vecname'_b '
                  matrix `vecname'_se`markse' = `vecname'_se'`markse'
                  }
            }
        }
        else {
            forvalues colj = 1/`colmax' {
              forvalues rowi = 1/`rowmax' {
                if abs(`vecname'_tbl[(`rowi')*2-1,`colj']/`vecname'_tbl[(`rowi')*2,`colj']) > abs(invnorm(`lvl1')) & abs(`vecname'_tbl[(`rowi')*2-1,`colj']/`vecname'_tbl[(`rowi')*2,`colj'])~=.  matrix `t_stars'[`rowi'*2-1,`colj']=1
                if abs(`vecname'_tbl[(`rowi')*2-1,`colj']/`vecname'_tbl[(`rowi')*2,`colj']) > abs(invnorm(`lvl2')) & abs(`vecname'_tbl[(`rowi')*2-1,`colj']/`vecname'_tbl[(`rowi')*2,`colj'])~=.  matrix `t_stars'[`rowi'*2-1,`colj']=2
                if abs(`vecname'_tbl[(`rowi')*2-1,`colj']/`vecname'_tbl[(`rowi')*2,`colj']) > abs(invnorm(`lvl3')) & abs(`vecname'_tbl[(`rowi')*2-1,`colj']/`vecname'_tbl[(`rowi')*2,`colj'])~=.  matrix `t_stars'[`rowi'*2-1,`colj']=3
                if `vecname'_tbl[(`rowi')*2,`colj']==. & substr("`mark'",1,2)=="it"  matrix `t_stars'[`rowi'*2-1,`colj']=9
              }
            }
            local starle = `rowmax'*2
            matrix `t_stars' = `t_stars'[1..`starle',1..`colmax']
        }
    }


    /* Transformation of master table to TeX */

    if `precision'~=0 & `precision' < .000001 {
      qui matsave `vecname'_tbl, path("`path'") t(double) `replace'
      }
    else {
      qui matsave `vecname'_tbl, path("`path'") t(float) `replace'
      }
    disp in green "matrix " in yellow "`file1'" in green " saved"
    local colname : colfullnames `vecname'_tbl
    local colmax = colsof(`vecname'_tbl)
    tokenize "`colname'"
    if `precision' ~= 0 {
        forvalues vari = 1/`colmax' {
        local strlen = `digits' + 2
          if `flexible' == 0 {
            qui gen str`strlen' s_``vari'' = ""
            qui replace ``vari'' = sign(``vari'') * int( (1/`precision') * abs(``vari'') + .5) * `precision' if ``vari''~=.
            }
          else {
           if `flexible' == 1 {
            local strlen = `strlen' + 1
            qui gen str`strlen' s_``vari'' = ""
            qui replace ``vari'' = sign(``vari'') * int( (1/`precision') * abs(``vari'') + .5) * `precision'       if ``vari''~=. & abs(``vari'') >= `precision'
            qui replace ``vari'' = sign(``vari'') * int( (10/`precision') * abs(``vari'') + .5) * `precision'/10   if ``vari''~=. & abs(``vari'') < `precision' & abs(``vari'') >=`precision'/10
           }
           else {
            local strlen = `strlen' + 2
            qui gen str`strlen' s_``vari'' = ""
            qui replace ``vari'' = sign(``vari'') * int( (1/`precision') * abs(``vari'') + .5) * `precision'       if ``vari''~=. & abs(``vari'') >= `precision'
            qui replace ``vari'' = sign(``vari'') * int( (10/`precision') * abs(``vari'') + .5) * `precision'/10   if ``vari''~=. & abs(``vari'') < `precision'     & abs(``vari'') >= `precision'/10
            qui replace ``vari'' = sign(``vari'') * int( (100/`precision') * abs(``vari'') + .5) * `precision'/100 if ``vari''~=. & abs(``vari'') < `precision'/10 & abs(``vari'') >= `precision'/100
           }
          }
        if "`dots'"=="dots" {
          qui replace s_``vari'' = "." if ``vari''==. & substr(trim(_rowname),`positse',3)~="_se" & substr(trim(_rowname),-4,4)~="_se`mse'"
          }
        else {
          qui replace s_``vari'' = "" if ``vari''==.
          }
        qui replace s_``vari'' = string(``vari'') if ``vari''~=.
        if "`fancy'" == "fancy" & "`plain'"=="" {
          qui replace s_``vari'' = string(``vari'') + "00000" if ( ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 5 & index(s_``vari'',".")~=0 ) | ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 5 & length(s_``vari'')>1 & substr(s_``vari'',1,1)=="." ) ) & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r("
          qui replace s_``vari'' = string(``vari'') + "0000" if ( ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 4 & index(s_``vari'',".")~=0 ) | ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 4 & length(s_``vari'')>1 & substr(s_``vari'',1,1)=="." ) ) & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r("
          qui replace s_``vari'' = string(``vari'') + "000" if ( ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 3 & index(s_``vari'',".")~=0 ) | ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 3 & length(s_``vari'')>1 & substr(s_``vari'',1,1)=="." ) ) & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r("
          qui replace s_``vari'' = string(``vari'') + "00" if ( ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 2 & index(s_``vari'',".")~=0 ) | ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 2 & length(s_``vari'')>1 & substr(s_``vari'',1,1)=="." ) ) & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r("
          qui replace s_``vari'' = string(``vari'') + "0" if ( ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 1 & index(s_``vari'',".")~=0 ) | ( ( length(s_``vari'') - index(s_``vari'',".") ) == `digits' - 1 & length(s_``vari'')>1 & substr(s_``vari'',1,1)=="." ) ) & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r("
          qui count if index(string(``vari''),".")~=0
          if `r(N)'>0 {
            qui replace s_``vari'' = string(``vari'') + ".`addzeros'" if index(s_``vari'',".")==0 & ``vari''~=. & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & substr(_rowname,-4,2)~="e(" & substr(_rowname,-4,2)~="r(" & `digits'~=0
            qui replace s_``vari'' = substr(s_``vari'',2,length(s_``vari'')) if substr(s_``vari'',1,2)=="0." 
            }
          if "`thousep'" == "thousep" {
            qui count if index(string(``vari''),".")~=0
            if `r(N)'==0 {
              qui replace s_``vari'' = substr(string(``vari''),-length(string(``vari'')),mod(length(string(``vari'')),3)) + "," + substr(string(``vari''),-12,3) + "," + substr(string(``vari''),-9,3) + "," + substr(string(``vari''),-6,3) + "," + substr(string(``vari''),-3,3) if index(s_``vari'',".")==0 & ``vari''~=. & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & length(string(``vari''))<15 & length(string(``vari''))>12
              qui replace s_``vari'' = substr(string(``vari''),-12,3) + "," + substr(string(``vari''),-9,3) + "," + substr(string(``vari''),-6,3) + "," + substr(string(``vari''),-3,3) if index(s_``vari'',".")==0 & ``vari''~=.  & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & length(string(``vari''))==12
              qui replace s_``vari'' = substr(string(``vari''),-length(string(``vari'')),mod(length(string(``vari'')),3)) + "," + substr(string(``vari''),-9,3) + "," + substr(string(``vari''),-6,3) + "," + substr(string(``vari''),-3,3) if index(s_``vari'',".")==0 & ``vari''~=.  & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & length(string(``vari''))<12 & length(string(``vari''))>9
              qui replace s_``vari'' = substr(string(``vari''),-9,3) + "," + substr(string(``vari''),-6,3) + "," + substr(string(``vari''),-3,3) if index(s_``vari'',".")==0 & ``vari''~=.  & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & length(string(``vari''))==9
              qui replace s_``vari'' = substr(string(``vari''),-length(string(``vari'')),mod(length(string(``vari'')),3)) + "," + substr(string(``vari''),-6,3) + "," + substr(string(``vari''),-3,3) if index(s_``vari'',".")==0 & ``vari''~=.  & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & length(string(``vari''))<9 & length(string(``vari''))>6
              qui replace s_``vari'' = substr(string(``vari''),-6,3) + "," + substr(string(``vari''),-3,3) if index(s_``vari'',".")==0 & ``vari''~=.  & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & length(string(``vari''))==6
              qui replace s_``vari'' = substr(string(``vari''),-length(string(``vari'')),mod(length(string(``vari'')),3)) + "," + substr(string(``vari''),-3,3) if index(s_``vari'',".")==0 & ``vari''~=.  & substr(_rowname,1,2)~="e(" & substr(_rowname,1,2)~="r(" & length(string(``vari''))<6 & length(string(``vari''))>3
              qui replace s_``vari'' = subinstr(s_``vari'',"-,","-",1)
              }
            }
        }
        if "`leadzero'" == "leadzero" & "`plain'"=="" {
          qui replace s_``vari'' = "0" + string(``vari'') if length(s_``vari'')>1 & substr(s_``vari'',1,1)=="."
          }
        qui drop ``vari''
        }
    }
    else {
        local strlen = 9
        forvalues vari = 1/`colmax' {
            qui gen str`strlen' s_``vari'' = string(``vari'') if ``vari''~=.
            if "`dots'"=="dots"  qui replace s_``vari'' = "." if ``vari''==.
            else  qui replace s_``vari'' = "" if ``vari''==.
        qui drop ``vari''
        }
    }

    local incr = "r"
    local explic = "l"
    
    forvalues vari = 1/`colmax' {
        qui gen str7 _stars`vari' = ""
        }
    if substr("`mark'",1,4)=="star" | substr("`mark'",1,2)=="it" {
      svmat byte `t_stars', names(eqcol)
      local renmvar : colfullnames `t_stars'
      foreach var in `renmvar' {
        local nwnm = subinstr("`var'",":","_",.)
        local olnm = subinstr("`var'",":","",.)
        capture rename _`olnm' `olnm'
        capture rename `olnm' `nwnm'
        }
      }
    if substr("`mark'",1,4)=="star"  local incr = "c"
    if substr("`mark'",1,2)=="it" & "`plain'"=="" {
      forvalues vari = 1/`colmax' {
        qui replace s_``vari'' = "\it " + s_``vari'' if ``vari''==0 & substr(trim(_rowname),`positse',3)~="_se" & ( s_``vari''~="." & s_``vari''~="" )
        }  
      }
    if substr("`mark'",1,5) == "stars" {
      forvalues vari = 1/`colmax' {
        qui replace _stars`vari' = "$^{*}$" if ``vari''[_n-`linesup']>=1 & substr(trim(_rowname),`positse',3)=="_se" & _n-`linesup'>0
        if `lvl2' < `lvl1'  qui replace _stars`vari' = "$^{**}$"  if ``vari''[_n-`linesup']>=2 & substr(trim(_rowname),`positse',3)=="_se" & _n-`linesup'>0
        if `lvl3' < `lvl1' & `lvl3' < `lvl2'  qui replace _stars`vari' = "$^{***}$" if ``vari''[_n-`linesup']>=3 & substr(trim(_rowname),`positse',3)=="_se" & _n-`linesup'>0
        }
      }
    if substr("`mark'",1,5) == "starb" {
      forvalues vari = 1/`colmax' {
        qui replace _stars`vari' = "$^{*}$" if ``vari''>=1 & substr(trim(_rowname),`positse',3)~="_se"
        if `lvl2' < `lvl1'  qui replace _stars`vari' = "$^{**}$"  if ``vari''>=2 & substr(trim(_rowname),`positse',3)~="_se"
        if `lvl3' < `lvl1' & `lvl3' < `lvl2'  qui replace _stars`vari' = "$^{***}$" if ``vari''>=3 & substr(trim(_rowname),`positse',3)~="_se"
        qui replace s_``vari'' = s_``vari'' + _stars`vari'  if substr(trim(_rowname),`positse',3)~="_se" & substr(trim(_rowname[_n+1]),`positse',3)=="_se" & ( s_``vari''~="." & s_``vari''~="" )
        }
      }
    if "`plain'"=="" {
      forvalues vari = 1/`colmax' {
        if "`multse'"==""  qui replace s_``vari'' = "\raisebox{.7ex}[0pt]{\scriptsize (" + s_``vari'' + ")" + _stars`vari' + "}" if substr(trim(_rowname),`positse',3)=="_se" & ( s_``vari''~="." & s_``vari''~="" )
        else {
            local msi = 0
            foreach mse in `multse' {
                local msi = `msi' + 1
                local raise = .7 + 1.0*(`msi'-1)
                qui replace s_``vari'' = "\raisebox{`raise'ex}[0pt]{\scriptsize (" + s_``vari'' + ")" + _stars`vari' + "}" if substr(trim(_rowname),-4,4)=="_se`mse'" & ( s_``vari''~="." & s_``vari''~="" )
            }        
        }
      }
      forvalues vari = 1/`colmax' {
          if `vari' < `colmax'   qui replace s_``vari'' = s_``vari'' + " &"
          if `vari' == `colmax'  qui replace s_``vari'' = s_``vari'' + " \" + "\" if _n < _N
          local explic = "`explic'" + "`incr'"
          if substr("`mark'",1,4)=="star" | substr("`mark'",1,2)=="it"  qui drop ``vari''
      }
      foreach vari in `extracols' {
          if `vari' < `colmax'  qui replace s_``vari'' = s_``vari'' + "&"
          local explic = "`explic'" + "`incr'"
      }
    } 
    else {  /* if option plain is chosen: */
      forvalues vari = 1/`colmax' {
        if "`multse'"==""  qui replace s_``vari'' = "(" + s_``vari'' + ")" + _stars`vari' if substr(trim(_rowname),`positse',3)=="_se" & ( s_``vari''~="." & s_``vari''~="" )
        else {
            foreach mse in `multse' {
                qui replace s_``vari'' = "(" + s_``vari'' + ")" + _stars`vari' if substr(trim(_rowname),-4,4)=="_se`mse'" & ( s_``vari''~="." & s_``vari''~="" )
            }        
        }
      }
    }
    forvalues vari = 1/`colmax' {
        drop _stars`vari'
        }
    
    if "`vallabels'"~="" & "`ready'"=="ready" {
      qui gen _introwname = int(real(_rowname))
      qui do "`path'`vallabels'"
      local lbl = subinstr("`vallabels'",".do","",.)
      label values _introwname `lbl'
      qui decode _introwname, gen(_newrowname)
      qui replace _newrowname = _rowname + " " + _newrowname
      qui drop _introwname _rowname
      rename _newrowname _rowname
      order _rowname
      }

    if "`suppress'"=="suppress" {
        if "`multse'"==""  qui drop if substr(trim(_rowname),`positse',3)=="_se"
        else  qui drop if substr(trim(_rowname),-4,3)=="_se"
        }
    else {
        if "`multse'"==""  qui replace _rowname = " " if substr(trim(_rowname),`positse',3)=="_se"
        else  qui replace _rowname = " " if substr(trim(_rowname),-4,3)=="_se"
        }
    
    qui replace _rowname = "cons" if _rowname == "_cons"
    if "`label'"=="label" { 
      qui replace _rowname = "Const." if _rowname == "cons" 
      } 
    qui replace _rowname = subinstr(_rowname,"_","-",length(_rowname))
    if "`plain'"=="" {
      qui replace _rowname = _rowname + " &"
      }


    /* Layout */
    
    if "`fancy'" ~= "fancy" & "`plain'"=="" {
        qui gen byte _line = _n
        order _line
        qui expand 3 in 1
        quietly summ _line
        qui expand 2 if _line==`r(max)'
        sort _line
        qui drop _line
        qui replace _rowname = "\begin{tabular}{`explic'}" if _n == 1
        qui replace _rowname = "" if _n == 2
        local vardsp = subinstr("`1'","_","-",length("`1'"))
        if "`collabels'"~="" & "`labl1'"~=""  local vardsp = "`labl1'"
        qui replace s_`1' = "& `vardsp' &" if _n == 2
        forvalues vari = 1/`colmax' {
            qui replace s_``vari'' = "" if _n == 1
            local vardsp = subinstr("``vari''","_","-",length("``vari''"))
            if "`collabels'"~="" & "`labl`vari''"~=""  local vardsp = "`labl`vari''"
            if `vari'>1 & `vari'<`colmax'  qui replace s_``vari'' = "`vardsp' &" if _n == 2
            qui replace s_``vari'' = "" if _n == _N
        }
        local vardsp = subinstr("``colmax''","_","-",length("``colmax''"))
        if "`collabels'"~="" & "`labl`colmax''"~=""  local vardsp = "`labl`colmax''"
        qui replace s_``colmax'' = "`vardsp' \" + "\" if _n == 2
        qui replace _rowname = "\end{tabular}%" if _n == _N
        foreach vari in `extracols' {
            if `vari' < `colmax'   qui replace s_``vari'' = s_``vari'' + "&" if _n == 2
        }
    }

    if "`fancy'" == "fancy" & "`plain'"=="" {
        qui gen int _line = _n
        order _line
        qui expand 6 in 1
        quietly summ _line
        qui expand 3 if _line==`r(max)'
        sort _line
        qui drop _line
        qui replace _rowname = "\begin{tabular*}{\textwidth}{@{\extracolsep{\fill}}`explic'}" if _n == 1
        qui replace _rowname = "" if _n == 2
        local vardsp = subinstr("`1'","_","-",length("`1'"))
        if "`collabels'"~="" & "`labl1'"~=""  local vardsp = "`labl1'"
        qui replace s_`1' = "& \multicolumn{1}{`incr'}{`vardsp'} &" if _n == 2
        local coltabl = `colmax' + 1 + `toknum'
        qui replace _rowname = "\cline{2-`coltabl'}" if _n == 3
        qui replace _rowname = "" if _n == 4
        qui replace s_`1' = "& \multicolumn{1}{`incr'}{(1)\mbox{\ }} &" if _n == 4
        qui replace _rowname = "\hline\"+"\"+"[-10pt]" if _n == 5
        forvalues vari = 1/`colmax' {
            qui replace s_``vari'' = "" if _n == 1
            qui replace s_``vari'' = "" if _n == 3
            qui replace s_``vari'' = "" if _n == 5
            if `vari'>1 & `vari'<`colmax' {
                local vardsp = subinstr("``vari''","_","-",length("``vari''"))
                if "`collabels'"~="" & "`labl`vari''"~=""  local vardsp = "`labl`vari''"
                qui replace s_``vari'' = "\multicolumn{1}{`incr'}{`vardsp'} &" if _n == 2
                qui replace s_``vari'' = "\multicolumn{1}{`incr'}{(`vari')\mbox{\ }} &" if _n == 4
                }
            qui replace s_``vari'' = "" if _n == _N-1
            qui replace s_``vari'' = "" if _n == _N
        }
        local vardsp = subinstr("``colmax''","_","-",length("``colmax''"))
        if "`collabels'"~="" & "`labl`colmax''"~=""  local vardsp = "`labl`colmax''"
        qui replace s_``colmax'' = "\multicolumn{1}{`incr'}{`vardsp'} \" + "\" if _n == 2
        qui replace s_``colmax'' = "\multicolumn{1}{`incr'}{(`colmax')} \" + "\" if _n == 4
        qui replace s_``colmax'' = s_``colmax'' + " \" + "\" if _n == _N-2
        qui replace _rowname = " ["+"2pt"+"]"+"\hline\hline" if _n == _N-1
        qui replace _rowname = "\end{tabular*}%" if _n == _N
        foreach vari in `extracols' {
            if `vari' < `colmax'  {
              qui replace s_``vari'' = s_``vari'' + "&" if _n == 2
              qui replace s_``vari'' = s_``vari'' + "&" if _n == 4
              }
        }
        if "`label'"=="label" {
            qui replace _rowname = "Obs. &" if _rowname=="e(N) &" | _rowname=="r(N) &"
            qui replace _rowname = "$ R^2$ &" if _rowname=="e(r2) &"
            qui replace _rowname = "$ R^2$ (overall) &" if _rowname=="e(r2-o) &"
            qui replace _rowname = "$ R^2$ (between) &" if _rowname=="e(r2-b) &"
            qui replace _rowname = "$ R^2$ (within) &" if _rowname=="e(r2-w) &"
            qui replace _rowname = "$ F$ statistic &"      if _rowname=="e(F) &"    | _rowname=="r(F) &"
            qui replace _rowname = "$ \chi^2$ statistic &" if _rowname=="e(chi2) &" | _rowname=="r(chi2) &"
            qui replace _rowname = "Degrees of freedom &"  if _rowname=="e(df) &"   | _rowname=="r(df) &"
            qui replace _rowname = "$ p$-value &"          if _rowname=="r(p) &"
            qui replace _rowname = "Log likelihood &"      if _rowname=="e(ll-r) &"   | _rowname=="r(ll-r) &"
            qui replace _rowname = "Model degrees of freedom &"    if _rowname=="e(df-m) &"  | _rowname=="r(df-m) &"
            qui replace _rowname = "Residual degrees of freedom &" if _rowname=="e(df-r) &"  | _rowname=="r(df-r) &"
            qui replace _rowname = "Model sum of squares &"        if _rowname=="e(mss) &"   | _rowname=="r(mss) &"
            qui replace _rowname = "Residual sum of squares &"     if _rowname=="e(rss) &"   | _rowname=="r(rss) &"
        }
    qui drop if substr(s_``colmax'',-1,1)~="\" & substr(_rowname,1,1)~="\" & substr(_rowname,1,1)~=" " & _rowname~=""
    }

    if "`plain'"~="" {
        qui gen byte _line = _n
        order _line
        qui expand 2 in 1
        sort _line
        qui replace _rowname = "" if _n == 1
        forvalues vari = 1/`colmax' {
            local vardsp = subinstr("``vari''","_","-",length("``vari''"))
            qui replace s_``vari'' = "`vardsp'" if _n == 1
        }
    }
    if "`label'"=="label" & "`lblnum'"=="" {  /* MH 16jan2002 */
        local i = 1
        local rowmax = rowsof(`vecname'_b)
        tokenize `vars'
        while `i'<=`rowmax' {
            qui replace _rowname = "`L`i'' &" if _rowname=="``i'' &"
            local i = `i'+1
            }
        }
    if "`lblnum'"~="" {
      forvalues l = 1/`lblnum' {
        qui replace _rowname = "`L`l'' &"  if  _rowname=="`V`l'' &"
        }
      forvalues l = 1/`lblnum' {
        qui replace _rowname = substr(_rowname,1,index(_rowname,":")) + "`L`l'' &"  if  _rowname==substr(_rowname,1,index(_rowname,":")) + "`V`l'' &"
        qui replace _rowname = "`L`l''" + substr(_rowname,index(_rowname,":"),length(_rowname)) if  _rowname=="`V`l''" + + substr(_rowname,index(_rowname,":"),length(_rowname))
        }
      }

    if "`plain'"==""    {
      capture outsheet using "`file2'", noquote nonames replace
      }
    if "`plain'"=="csv" {
      capture outsheet using "`file2'", noquote nonames comma replace
      }
    if "`plain'"=="tab" {
      capture outsheet using "`file2'", noquote nonames replace
      }
    if _rc==0 {
      disp in green "TeX file " in yellow "`file2'" in green " saved"
      }
    else {
      use "`file1'", clear
      error _rc
      }

    use "`file1'", clear
end

capture program drop est2vec
program define est2vec
    version 8
    args matini
    local matini = subinstr("`matini'",",","",1)
    capture syntax newvarname [, Vars(string) SHVars(varlist) E(string) R(string) ADDto(string) RADDto(string) NAme(string) FORCE noKEEP REPLACE ALTVec(string) ]
    if _rc==111 {
        syntax [newvarname] [, Vars(string) SHVars(string) E(string) R(string) ADDto(string) RADDto(string) NAme(string) FORCE noKEEP REPLACE ALTVec(string) ]
        foreach name of local shvars {
            local name = subinstr("`name'","-"," ",.)
            capture confirm variable `name'
            if _rc~=198  confirm variable `name'
            }
        }
    if _rc==100 {
        syntax [newvarname] [, Vars(string) SHVars(varlist) E(string) R(string) ADDto(string) RADDto(string) NAme(string) FORCE noKEEP REPLACE ALTVec(string) ]
        if "`addto'"~=""   local matini = "`addto'"
        if "`raddto'"~=""  local matini = "`raddto'"
        if "`matini'"=="" {
          disp as err "matrix name required"
          exit _rc
          }
        }
    if _rc~=0 & _rc~=100  error _rc

    if "`matini'"=="`addto'" || "`matini'"=="`raddto'"  local keep = "nokeep"

    /* Names of vectors */
    if "`keep'"=="nokeep" { /* temporary names */
      tempname `matini'_b
      tempname `matini'_se
      tempname `matini'_e
      tempname `matini'_r
      }
    else { /* permanent vectors */
      local `matini'_b  = "`matini'_b"
      local `matini'_se = "`matini'_se"
      local `matini'_e  = "`matini'_e"
      local `matini'_r  = "`matini'_r"
      }  

    if "`vars'"~=""  local shvars = ""
    if "`addto'"=="" & "`raddto'"=="" & "`replace'"~="replace" {
            tempname chk
            if "`r'" == "" {
              foreach type in b se e {
                capture local `chk' = colsof(``matini'_`type'')
                if _rc==0 {
                  disp as err "no; matrix ``matini'_`type'' would be lost"
                  exit 4
                  }
                }
              }
            else {
                capture local `chk' = colsof(``matini'_r')
                if _rc==0 {
                  disp as err "no; matrix ``matini'_r' would be lost"
                  exit 4
                  }
              }
        }
    if "`keep'"=="nokeep" & ("`addto'"=="" & "`raddto'"=="") {
            disp as err "nokeep without addto() or raddto() invalid"
            exit 198
            }
    if "`addto'"~="" & "`raddto'"~="" {
            disp as err "use of addto() and raddto() invalid"
            exit 198
            }
    if "`addto'"~="" {
        if "`vars'"~="" || "`shvars'"~="" || "`e'"~="" || "`r'"~="" {
            disp as err "use of addto() invalid in combination with other options"
            exit 198
            }
        tempname tst
        foreach type in b se e {
            local `tst' = colsof(`addto'_`type')
            }
        local colnames : colfullnames `addto'_b
        local colmax = colsof(`addto'_b)
        tokenize `colnames'
        forvalues i = 1/`colmax' {
            if "``i''"=="`name'" {
                disp as err "column name ``i'' not unique or already defined"
                exit 110
                }
            if "`name'"=="" & "``i''"=="`e(depvar)'" {
                disp as err "column name ``i'' not unique or already defined"
                exit 110
                }
            }
        }
    if "`raddto'"~="" {
        if "`vars'"~="" || "`shvars'"~="" || "`e'"~="" || "`r'"~="" {
            disp as err "use of raddto() invalid"
            exit 198
            }
        tempname tst
        local `tst' = colsof(`raddto'_r)
        local colnames : colfullnames `raddto'_r
        local colmax = colsof(`raddto'_r)
        tokenize `colnames'
        forvalues i = 1/`colmax' {
            if "``i''"=="`name'" {
                disp as err "column name ``i'' not unique or already defined"
                exit 110
                }
            if "`name'"=="" & "``i''"=="`e(depvar)'" {
                disp as err "column name ``i'' not unique or already defined"
                exit 110
                }
            }
        }
    if "`vars'"~="" & "`r'"~="" {
        disp as err "combination vars() and r() invalid"
        exit 198
        }
    if "`shvars'"~="" & "`r'"~="" {
        disp as err "combination shvars() and r() invalid"
        exit 198
        }
    if "`e'"~="" & "`r'"~="" {
        disp as err "combination e() and r() invalid"
        exit 198
        }
    if "`addto'"=="" & "`force'"=="force" {
            disp as err "use of force without addto() invalid"
            exit 198
            }
    if "`shvars'"~="" & "`force'"=="force" {
        disp as err "use of force with shvars() invalid"
        exit 198
        }
    if "`altvec'"~="" {
        local chk : word count `altvec'
        if `chk'~=2 {
          disp as err "option altvec() must specify two vectors, one for coefficients and one for standard errors"
          exit 198
          }
        tokenize `altvec'
        local altb  "`1'"
        local altse "`2'"
        }
    if "`name'"=="all" | "`name'"=="_all" {
        disp as err "specified column name " in yellow "`name'" in red " not permitted (system name conflict)"
        exit 198
        }


    /* Start working */

    local eqon = 0
    local roweq = ""
    if "`addto'"~="" {
        local vars : rowfullnames `addto'_b
        local e : rowfullnames `addto'_e
        local prveq : roweq `addto'_b
        tempname tmpchk
        matrix `tmpchk' = e(b)'
        local neweq : roweq `tmpchk'
        if "`neweq'" ~= ""  local eqon = 1
        }
    if "`raddto'"~=""  local r : rowfullnames `raddto'_r

    tempname tmpb tmpse tmpe tmpr
    if "`altvec'"~="" {
      tempname usealtb usealtse
      matrix `usealtb' = e(`altb')'
      matrix `usealtse' = e(`altse')'
      }
    if "`vars'" == "" & "`shvars'" == "" & "`r'"=="" {  /* If no vars spec.: use all regressors */
        matrix `tmpb' = e(b)'
        matrix `tmpse' = vecdiag(e(V))'
        local rownmb = rowsof(`tmpse')
        forvalues r = 1/`rownmb' {
          matrix `tmpse'[`r',1]=sqrt(`tmpse'[`r',1])
          }
        if "`altvec'"~="" {
          matrix `tmpb' = `usealtb'
          matrix `tmpse' = `usealtse'
          }
        }
    if "`vars'" ~= "" & "`r'"=="" {   /* If addto(), vars : rowfullnames `addto'_b */
        local varnum : word count `vars'
        matrix `tmpb'  = J(`varnum',1,.)
        matrix rownames `tmpb' = `vars'
        matrix `tmpse' = J(`varnum',1,.)
        matrix rownames `tmpse' = `vars'
        if "`force'" == "force" {
          matrix roweq `tmpb' = `neweq'
          matrix roweq `tmpse' = `neweq'
          local vars : rowfullnames `tmpb'
          }
        local i = 0
        foreach var of local vars {
            local i = `i' + 1
            tokenize "`var'", parse(":")
            if "`3'" == "" {
              local 3 = "`1'"
              local 1 = "#1"
              }
            capture matrix `tmpb'[`i',1]  =  [`1']_b[`3']
            capture matrix `tmpse'[`i',1] =  [`1']_se[`3']
            if _rc~=0 & _rc~=111 & _rc~=303  error _rc
            if "`altvec'"~="" {
              tempname usematb usematse
              capture matkeep `usealtb', matrix(`usematb') ifr(`var')
              if _rc  matrix `tmpb'[`i',1] = .
              else  matrix `tmpb'[`i',1] = `usematb'[1,1]
              capture matkeep `usealtse', matrix(`usematse') ifr(`var')
              if _rc  matrix `tmpse'[`i',1] = .
              else  matrix `tmpse'[`i',1] = `usematse'[1,1]
              }
            }
        }
    if "`shvars'" ~= "" & "`r'"=="" {
        local varnum : word count `shvars'
        local varnum = `varnum' + 1
        matrix `tmpb'  = J(`varnum',1,.)
        matrix rownames `tmpb' = `shvars' _cons
        matrix `tmpse' = J(`varnum',1,.)
        matrix rownames `tmpse' = `shvars' _cons
        local i = 0
        foreach var in `shvars' _cons {
            local i = `i' + 1
            capture matrix `tmpb'[`i',1]  = _b[`var']
            capture matrix `tmpse'[`i',1] = _se[`var']
            if _rc~=0 & _rc~=111 & _rc~=303  error _rc
            if "`altvec'"~="" {
              tempname usematb usematse
              capture matkeep `usealtb', matrix(`usematb') ifr(`var')
              if _rc  matrix `tmpb'[`i',1] = .
              else  matrix `tmpb'[`i',1] = `usematb'[1,1]
              capture matkeep `usealtse', matrix(`usematse') ifr(`var')
              if _rc  matrix `tmpse'[`i',1] = .
              else  matrix `tmpse'[`i',1] = `usematse'[1,1]
              }
            }
        }
    if "`r'"=="" {  /* Matrix replacement and naming, and treatment of e() */
        matrix ``matini'_b' = `tmpb'
        tokenize "`e(depvar)'"
        if "`name'"==""  matname ``matini'_b'  `1' , columns(1) explicit
        else             matname ``matini'_b'  `name' , columns(1) explicit
        matrix ``matini'_se' = `tmpse'
        if "`name'"==""  matname ``matini'_se' `1' , columns(1) explicit
        else             matname ``matini'_se' `name' , columns(1) explicit
        if `e(N)'~=.  matrix input `tmpe' = (`e(N)')
        else  matrix input `tmpe' = (.)
        matrix ``matini'_e' = `tmpe'
        matname ``matini'_e' "e(N)" , rows(1) explicit
        if "`name'"==""  matname ``matini'_e' `1' , columns(1) explicit
        else             matname ``matini'_e' `name' , columns(1) explicit
        foreach savres in `e' {  
           if "`addto'"~="" {
                local savres = subinstr("`savres'","e(","",1)
                local savres = subinstr("`savres'",")","",1)
                } 
           if "`savres'" ~= "N" {  
             matrix drop `tmpe'
             matrix input `tmpe' = (.)
             capture if `e(`savres')'~=.     matrix input `tmpe' = (`e(`savres')')
             capture if "`e(`savres')'"~=""  matrix input `tmpe' = (`e(`savres')')
             matrix ``matini'_e'=``matini'_e' \ `tmpe'
             local rowid = rowsof(``matini'_e')
             matname ``matini'_e' "e(`savres')" , rows(`rowid') explicit
             } 
            }
        if "`addto'"~="" {   /* Addto routine */
            foreach type in b se e {
                capture matrix `addto'_`type' = `addto'_`type' , ``matini'_`type''
                if _rc==503 {
                       disp as err "conformability error in rows of `addto'_`type' and `matini'_`type'"
                       exit _rc
                }
                if _rc~=0  error _rc
                }
            }
    }
    else {  /* Treat r() */
        tempname `matini'_nwr
        matrix ``matini'_nwr' = (.)
        matname ``matini'_nwr' "_todelete" , rows(1) explicit
        tokenize "`e(depvar)'"
        if "`name'"==""  matname ``matini'_nwr' `1' , columns(1) explicit
        else             matname ``matini'_nwr' `name' , columns(1) explicit
        foreach savres in `r' {
           if "`raddto'"~="" {
                local savres = subinstr("`savres'","r(","",1)
                local savres = subinstr("`savres'",")","",1)
                } 
             capture matrix drop `tmpr'
             matrix input `tmpr' = (.)
             capture if `r(`savres')'~=.     matrix input `tmpr' = (`r(`savres')')
             capture if "`r(`savres')'"~=""  matrix input `tmpr' = (`r(`savres')')
             matrix ``matini'_nwr'=``matini'_nwr' \ `tmpr'
             local rowid = rowsof(``matini'_nwr')
             matname ``matini'_nwr' "r(`savres')" , rows(`rowid') explicit
           }
        if "`raddto'"~="" {
                matrix ``matini'_nwr' = ``matini'_nwr'[2..rowsof(``matini'_nwr'),1]
                capture matrix `raddto'_r = `raddto'_r , ``matini'_nwr'
                if _rc~=0 {
                    if _rc==503 {
                        disp as err "conformability error in rows of `raddto'_r and `matini'_r"
                        exit _rc
                        }
                    else  error _rc
                }
            }
        else  matrix ``matini'_r' = ``matini'_nwr'[2..rowsof(``matini'_nwr'),1]
        matrix drop ``matini'_nwr'
    }
end

program define matins
  version 8
  syntax newvarname , Matrix(string) [ IFRow(string) IFCol(string) OVERwrite ]
  local matname = subinstr("`1'",`","',"",.)
  local cols = colsof(`matname')
  local rows = rowsof(`matname')
  tempname newmat
  * local newmat "newmat"
  if "`ifrow'"=="" & "`ifcol'"=="" {
    disp as err "row or column list required"
    exit 100
    }
  if "`ifrow'"~="" & "`ifcol'"~="" {
    disp as err "simultaneous row and column lists not allowed"
    exit 101
    }
  
  /* Row Keep and Order */
  if "`ifrow'"~="" {
    tempname addrow
    mat `addrow' = J(1,colsof(`matname'),.)
    * check existence of rows
    local rownames : rowfullnames `matname'
    local totrow : word count `rownames'
    tokenize `rownames'
    foreach row in `ifrow' {
      mat rownames `addrow' = `row'
      capture local chk = rowsof(`newmat')
      if _rc~=0 {
        if rownumb(`matname',"`row'")~=. {
          mat `newmat' = `matname'[rownumb(`matname',"`row'"),1..colsof(`matname')]
          }
        else {
          mat `newmat' = `addrow'
          }
        }
      else {
        if rownumb(`matname',"`row'")~=. {
          mat `newmat' = `newmat' \ `matname'[rownumb(`matname',"`row'"),1..colsof(`matname')]
          }
        else {
          mat `newmat' = `newmat' \ `addrow'
          }
        }
      }
    }
  
  /* Col Keep and Order */
  if "`ifcol'"~="" {
    tempname addcol
    mat `addcol' = J(rowsof(`matname'),1,.)
    * check existence of columns
    local colnames : colfullnames `matname'
    local totcol : word count `colnames'
    tokenize `colnames'
    foreach col in `ifcol' {
      mat colnames `addcol' = `col'
      capture local chk = colsof(`newmat')
      if _rc~=0 {
        if colnumb(`matname',"`col'")~=. {
          mat `newmat' = `matname'[1..rowsof(`matname'),colnumb(`matname',"`col'")]
          }
        else {
          mat `newmat' = `addcol'
          }
        }
      else {
        if colnumb(`matname',"`col'")~=. {
          mat `newmat' = `newmat' , `matname'[1..rowsof(`matname'),colnumb(`matname',"`col'")]
          }
        else {
          mat `newmat' = `newmat' , `addcol'
          }
        }
      }
    }

  /* Finalize */
  if "`matrix'"=="`matname'" {
    mat drop `matname'
    mat `matname' = `newmat'
    }
  else {
    capture local check = colsof(`matrix')
    if _rc==0 & "`overwrite'"=="" {
       disp as err "no; matrix " in yellow "`matrix'" in red " would be lost"
       exit 4 
       }
    mat `matrix' = `newmat'
    }

end

program define matkeep
  version 7
  syntax newvarname , Matrix(string) [ IFRow(string) IFCol(string) OVERwrite ]
  local matname = subinstr("`1'",`","',"",.)
  local cols = colsof(`matname')
  local rows = rowsof(`matname')
  tempname newmat
  * local newmat "newmat"
  if "`ifrow'"=="" & "`ifcol'"=="" {
    disp as err "row or column list required"
    exit 100
    }
  if "`ifrow'"~="" & "`ifcol'"~="" {
    disp as err "simultaneous row and column lists not allowed"
    exit 101
    }
  
  /* Row Keep */
  if "`ifrow'"~="" {
  local totrows :  word count `ifrow'
  tokenize `ifrow'
  local rownames : rowfullnames `matname'
  mat `newmat' = `matname'[1,1..`cols']  /* keep first row for build-up */
  local pickrow = 0
  foreach rowname in `rownames' {
    local pickrow = `pickrow' + 1
    forvalues i = 1/`totrows' {
      if "`rowname'"=="``i''" {  /* add row by row */
        mat `newmat' = `newmat' \ `matname'[`pickrow',1..`cols']
        }
      }
    }
  if rowsof(`newmat') > 1 {
    mat `newmat' = `newmat'[2..`totrows'+1,1..`cols']  /* remove first row */
    }
  else {
    disp as err "specified row(s) not found"    
    exit 111
    }
  }
  
  /* Col Keep */
  if "`ifcol'"~="" {
  local totcols :  word count `ifcol'
  tokenize `ifcol'
  local colnames : colfullnames `matname'
  mat `newmat' = `matname'[1..`rows',1]  /* keep first column for build-up */
  local pickcol = 0
  foreach colname in `colnames' {
    local pickcol = `pickcol' + 1
    forvalues i = 1/`totcols' {
      if "`colname'"=="``i''" {  /* add column by column */
        mat `newmat' = `newmat' , `matname'[1..`rows',`pickcol']
        }
      }
    }
  if colsof(`newmat') > 1 {
    mat `newmat' = `newmat'[1..`rows',2..`totcols'+1]  /* remove first column */
    }
  else {
    disp as err "specified column(s) not found"    
    exit 111
    }
  }

  /* Finalize */
  if "`matrix'"=="`matname'" {
    mat drop `matname'
    mat `matname' = `newmat'
    }
  else {
    capture local check = colsof(`matrix')
    if _rc==0 & "`overwrite'"=="" {
       disp as err "no; matrix " in yellow "`matrix'" in red " would be lost"
       exit 4 
       }
    mat `matrix' = `newmat'
    }
end

program define matload
    version 7
    args matname
    capture syntax newvarname [, Saving DROPALL Path(string) Missing(string) ROWname(string) OVERwrite]
    if _rc==100 {
       disp as err "matrix and file name required" 
       exit _rc
       }
    if _rc~=0 { 
      error _rc 
      exit
      }
    if "`missing'"==""  { local missing = -999 }
    local missing = int(real("`missing'"))

    global err_mssg = ""
    local rc 0
    local matname= subinstr("`matname'",",","",1)
    if "$S_OS" == "MacOS" {
      local dirsep = ":"
      }
    else {
      local dirsep = "/"
      }
    if (substr("`path'",-1,1)~="\" & substr("`path'",-1,1)~="/" & substr("`path'",-1,1)~=":") & "`path'"~="" {
      local path "`path'`dirsep'"
      }
    local currN = _N
    if `currN'>0 & "`saving'"~="saving" & "`dropall'"~="dropall" {
             error 4
             exit
             }
    local file "`path'`matname'.dta"
    confirm file "`file'"
    tempname chk
    capture local `chk' = colsof(`matname')
    if _rc==0 & "`overwrite'"=="" {
               disp as err "no; matrix " in yellow "`matname'" in red " would be lost"
               exit 4 
               }
     local saved 0
     if "`saving'"=="saving" & `currN'>0 {
       local dropall = ""
       tempfile tmp
       quietly save `tmp'
       local saved 1
       disp in green "data in memory will be removed temporarily, and restored later"
       disp in green "Press any key to continue, or Break to abort"
       more
       drop _all
       }
     if "`dropall'"=="dropall" & `currN'>0 {
       disp in green "data in memory will be dropped"
       disp in green "Press any key to continue, or Break to abort"
       more
       drop _all
       }
    capture use "`file'", clear
        if "`rowname'"=="" {
            capture confirm variable _rowname
            if _rc~=0 {local rc = _rc
                       global err_mssg = "_rowname not found"}
            qui count
            if `r(N)'==0 {local rc = _rc
                          global err_mssg = "Data set empty or not Stata format"}
            }
        else {
            capture confirm variable `rowname'
            if _rc~=0 {local rc = _rc
                       global err_mssg = "`rowname' not found"}
            else {rename `rowname' _rowname}
            qui count
            if `r(N)'==0 {local rc = _rc
                          global err_mssg = "Data set empty or not Stata format"}
            }
        capture confirm new variable `matname'
        if _rc~=0 {local rc = _rc
                   global err_mssg = "matrix `matname' contains variable `matname'"}
    if _rc==0 {
        capture {
        local j 1
            while `j' <= _N {
                local rownm`j'=_rowname[`j']
                local j = `j'+1
            }
        drop _rowname
        }
        if _rc~=0 & "${err_mssg}" == "" {
                   local rc = _rc
                   global err_mssg = "error (before mkmat was applied)"
                   }
        if `missing'~=15127 & _N > 0 { 
            foreach var of varlist * { 
                capture qui replace `var'=`missing' if `var'==. 
                if _rc~=0 & "${err_mssg}" == "" {
                           local rc = _rc
                           global err_mssg = "file `matname'.dta contains string variables"
                           }
                }
            } 
        capture mkmat _all, matrix(`matname')
        if _rc~=0 & "${err_mssg}" == "" {
                    local rc = _rc
                    global err_mssg = "error (as mkmat was applied)"
                    }
        capture {
        local j 1
        while `j' <= _N {
            matname `matname' `rownm`j'', rows(`j') explicit
            local j = `j'+1
            }
        local cnam : colfullnames `matname'
        tokenize "`cnam'"
        local j 1
        while `j' <= colsof(`matname') {
            local `j'=subinword("``j''","__cons","_cons",1)
            local `j'=subinword("``j''","__b","_b",1)
            local `j'=subinword("``j''","__coef","_coef",1)
            matname `matname' ``j'' , columns(`j') explicit
          local j=`j'+1
        }
        drop _all
        }
    }
    if _rc~=0 & "${err_mssg}" == "" {
                local rc = _rc
                global err_mssg = "error (after mkmat was applied)"
                }
    if _rc==0 & `rc'==0 {disp in green "matrix " in yellow "`matname'" in green " loaded"}
    if `saved' {use `tmp', clear
                disp in green "data in memory restored"
                }
    if `rc'~=0 {
                disp as err "${err_mssg}"
                error `rc'
                }
    global err_mssg = ""
    exit `rc'
end

program define matmerge
  version 8.2
  syntax newvarname , Matrix(string) [ ASRow ASCol ]
  local matname = subinstr("`1'",`","',"",.)
  local cols = colsof(`matname')
  local rows = rowsof(`matname')
  tempname newmat
  * local newmat "newmat"
  if "`asrow'"=="" & "`ascol'"=="" {
    disp as err "row or column merge must be specified"
    exit 100
    }
  if "`ifrow'"~="" & "`ifcol'"~="" {
    disp as err "simultaneous row and column merge not allowed"
    exit 101
    }
  capture local chk = rowsof(`matrix')
  if _rc~=0 {
    mat `matrix' = `matname'
    exit
    }
  if "`matrix'"=="`matname'" {
    if "`asrow'"~="" { 
      mat `matrix' = `matrix' \ `matname'
      }
    if "`ascol'"~="" { 
      mat `matrix' = `matrix' , `matname'
      }
    exit
    }
  local colsold = colsof(`matrix')
  local rowsold = rowsof(`matrix')
  
  /* Row Merge */
  if "`asrow'"~="" {
    local rownames : rowfullnames `matname'
    local rowno = rowsof(`matname')
    local colvars : colfullnames `matrix'
    forvalues i = 1/`rowno' {
      mat `matrix' = `matrix' , J(1,`colsold',.)
      local newrow = rowsof(`matrix')
      tokenize `rownames'
      matname `matrix' ``i'', rows(`newrow') explicit
      tokenize `colvars'
      forvalues colj = 1/`colsold' {
        mat `matrix'[`newrow',`colj'] = `matname'[`i',colnumb(`matname',"``colj''")]
        if `matrix'[`newrow',`colj']==. { 
          mat `matrix'[`newrow',`colj'] = .
          }
        }
      }
    }

  /* Col Merge */
  if "`ascol'"~="" {
    local colnames : colfullnames `matname'
    local colno = colsof(`matname')
    local rowvars : rowfullnames `matrix'
    forvalues j = 1/`colno' {
      mat `matrix' = `matrix' , J(`rowsold',1,.)
      local newcol = colsof(`matrix')
      tokenize `colnames'
      matname `matrix' ``j'', columns(`newcol') explicit
      tokenize `rowvars'
      forvalues rowi = 1/`rowsold' {
        mat `matrix'[`rowi',`newcol'] = `matname'[rownumb(`matname',"``rowi''"),`j']
        if `matrix'[`rowi',`newcol']==. { 
          mat `matrix'[`rowi',`newcol'] = .
          }
        }
      }
    }

end

program define matorder
  version 7
  syntax newvarname , Matrix(string) [ IFRow(string) IFCol(string) OVERwrite ]
  local matname = subinstr("`1'",`","',"",.)
  local cols = colsof(`matname')
  local rows = rowsof(`matname')
  tempname newmat
  * local newmat "newmat"
  if "`ifrow'"=="" & "`ifcol'"=="" {
    disp as err "row or column list required"
    exit 100
    }
  if "`ifrow'"~="" & "`ifcol'"~="" {
    disp as err "simultaneous row and column lists not allowed"
    exit 101
    }
  
  /* Row Keep and Order */
  if "`ifrow'"~="" {
    * check existence of rows
    local rownames : rowfullnames `matname'
    local totrow : word count `rownames'
    tokenize `rownames'
    foreach row in `ifrow' {
      local find = 0
      forvalues i = 1/`totrow' {
        if "`row'"=="``i''" {
          local find = `find' + 1
          }
        }
      if `find' == 0 {
        disp as err "row " in yellow "`row'" in red " not found in matrix " in yellow "`matname'"
        exit 111
        }
      }
    * work on order 
    foreach row in `ifrow' {
      capture local chk = rowsof(`newmat')
      if _rc~=0 {
        mat `newmat' = `matname'[rownumb(`matname',"`row'"),1..colsof(`matname')]
        }
      else {
        mat `newmat' = `newmat' \ `matname'[rownumb(`matname',"`row'"),1..colsof(`matname')]
        }
      }
    }
  
  /* Col Keep and Order */
  if "`ifcol'"~="" {
    * check existence of columns
    local colnames : colfullnames `matname'
    local totcol : word count `colnames'
    tokenize `colnames'
    foreach col in `ifcol' {
      local find = 0
      forvalues i = 1/`totcol' {
        if "`col'"=="``i''" {
          local find = `find' + 1
          }
        }
      if `find' == 0 {
        disp as err "column " in yellow "`col'" in red " not found in matrix " in yellow "`matname'"
        exit 111
        }
      }
    * work on order
    foreach col in `ifcol' {
      capture local chk = colsof(`newmat')
      if _rc~=0 {
        mat `newmat' = `matname'[1..rowsof(`matname'),colnumb(`matname',"`col'")]
        }
      else {
        mat `newmat' = `newmat' , `matname'[1..rowsof(`matname'),colnumb(`matname',"`col'")]
        }
      }
    }

  /* Finalize */
  if "`matrix'"=="`matname'" {
    mat drop `matname'
    mat `matname' = `newmat'
    }
  else {
    capture local check = colsof(`matrix')
    if _rc==0 & "`overwrite'"=="" {
       disp as err "no; matrix " in yellow "`matrix'" in red " would be lost"
       exit 4 
       }
    mat `matrix' = `newmat'
    }

end

program define matsave

     version 7
     global err_mssg = ""
     local rc 0
     args matname
     capture syntax newvarname [, REPLACE Saving DROPALL Path(string) Type(string)]

     if _rc==100 {
        disp as err "matrix name required" 
        exit _rc}
     if _rc~=0 {error _rc
                exit}

     local matname= subinstr("`matname'",",","",1)

     if "$S_OS" == "MacOS" {
       local dirsep = ":"
       }
     else {
       local dirsep = "/"
       }  
     if (substr("`path'",-1,1)~="\" & substr("`path'",-1,1)~="/" & substr("`path'",-1,1)~=":") & "`path'"~="" {
       local path "`path'`dirsep'"
       }
     if lower("`type'")~="" & lower("`type'")~="byte" & lower("`type'")~="int" & lower("`type'")~="long" & lower("`type'")~="float" & lower("`type'")~="double" {
       di in green "storage type reset to " in yellow "float"
       local type = "float"
       }

     local colmax = colsof(`matname')
     local collabels : colfullnames `matname'
     local chk : list dups collabels
     if "`chk'"~="" {
       disp as err "columns of " in yellow "`matname'" in red " are not uniquely named"
       error 110
       }

     local currN = _N
     if `currN'>0 & "`saving'"~="saving" & "`dropall'"~="dropall" {
       error 4
       exit
       }
     local file "`path'`matname'.dta"
     if "`replace'"~="replace" {
       confirm new file "`file'"
       }
     local saved 0

     if "`saving'"=="saving" & `currN'>0 {
       local dropall = ""
       tempfile tmp
       quietly save `tmp'
       local saved 1
       disp in green "data in memory will be removed temporarily, and restored later"
       disp in green "Press any key to continue, or Break to abort"
       more
       drop _all
       }
     if "`dropall'"=="dropall" & `currN'>0 {
       disp in green "data in memory will be dropped"
       disp in green "Press any key to continue, or Break to abort"
       more
       drop _all
       }

     capture {
     local chgflg =0
     local cnam : colfullnames `matname'
     tokenize `cnam'
     local fstwarn 1
     local i 1
     while `i' <= colsof(`matname') {
        if "``i''"~=subinstr("``i''",":","_",.) | "``i''"~=subinstr("``i''","(","",.) | "``i''"~=subinstr("``i''",")","",.) | "``i''"~=subinstr("``i''","[","",.) | "``i''"~=subinstr("``i''","]","",.) {
           local `i' = subinstr("``i''",":","_",.)
           local `i' = subinstr("``i''","(","",.)
           local `i' = subinstr("``i''",")","",.)
           local `i' = subinstr("``i''","[","",.)
           local `i' = subinstr("``i''","]","",.)
           if `fstwarn'==1 {
             noi disp in green "inadmissible ascii signs removed from column names of " in yellow "`matname'"
             local fstwarn 0
             }
           }
         if "``i''" == "_cons" | "``i''" == "_b" | "``i''" == "_coef" {
            local chgflg 1
            local `i' = "_" + "``i''"
            }
         matname `matname' :``i'', columns(`i') explicit
         local i = `i' + 1   
         }  
     local dosv=1
     * set more off
     }
     if _rc~=0 {local rc = _rc
                global err_mssg = "error (before svmat was applied)"}

     capture svmat `type' `matname', names(col)
     if _rc~=0 & "${err_mssg}" == "" {
                local rc = _rc
                global err_mssg = "error (as svmat was applied)"}

     capture {
     * set more on
     local i 1
     if `chgflg' {
         while `i' <= colsof(`matname') {
             matname `matname' ``i'', columns(`i') explicit
             local i = `i' +1   
             }  
     }  

     local rnam : rowfullnames `matname'
     tokenize "`rnam'"
     local maxlen= 0
     local j 1
     while `j' <= rowsof(`matname') {
         if length("``j''") > `maxlen' {local maxlen = length("``j''")}
         local j=`j'+1
         }
     if `maxlen' >80 {local maxlen = 80}
     quietly gen str`maxlen' _rowname=""
     local j 1
     while `j' <= rowsof(`matname') {
         quietly replace _rowname = "``j''" in `j'
         local j=`j'+1
         }
     order _rowname

     if "`replace'"=="replace" {save "`file'", replace}
     else {save "`file'"}
     }
     if _rc~=0 & "${err_mssg}" == "" {
                local rc = _rc
                global err_mssg = "error (after svmat was applied)"}
     if _rc==0 & `rc'==0 {disp in green "matrix " in yellow "`matname'" in green " saved"}
     if `saved' {use `tmp', clear
                 disp in green "data in memory restored"}
     if `rc' == 110 {
        disp as error "new variables cannot be uniquely named or already defined"
        error `rc'
        exit
     }

     if `rc'~=0 {disp as err "${err_mssg}"}
     global err_mssg = ""
     error `rc'

end
